/**
 * Ono Project
 *
 * File:         OnoMessage.java
 * RCS:          $Id: OnoMessage.java,v 1.2 2007/12/26 18:13:58 drc915 Exp $
 * Description:  OnoMessage class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 21, 2007 at 9:34:35 AM
 * Language:     Java
 * Package:      edu.northwestern.ono.net
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.net;

import java.nio.ByteBuffer;

import edu.northwestern.ono.util.NewTagByteBuffer;


/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The OnoMessage class ...
 */
public abstract class OnoMessage {
	
    /** message type constant for requesting a list of edges */
    public static final byte TYPE_EDGES = 1;

    /** message type for data sent directly between source and dest */
    public static final byte TYPE_PRIMARY_PATH = 2;

    /** message type for race between direct and one-hop path */
    public static final byte TYPE_RACE = 3;

    /** message type for receiver sending race result */
    static public final byte TYPE_RACE_RESULT = 4;

    /** message type for list of edges */
    public static final byte TYPE_EDGE_LIST = 5;

    /** message type for requesting peer list from dest */
    public static final byte TYPE_PEER_REQUEST = 6;

    /** message type for sending peer list to source */
    public static final byte TYPE_PEER_LIST = 7;

    // TODO send acks along paths their data came from...
    /** message type for message-receipt acknowledgement */
    public static final byte TYPE_ACK = 8;
    
    /** message type sent to dest to announce an incoming experiment */
    public static final byte TYPE_EXP_ANNOUNCE = 9;
    
    /** message type for sending destination for middle node to connect to */
    public static final byte TYPE_DEST_IP = 10;
    

	protected byte type;
	protected NewTagByteBuffer tbb;
	protected ByteBuffer bb;
	
	public static OnoMessage createMessage(NewTagByteBuffer data){
		data.getBuffer().position(0);
		byte type = data.getBuffer().get();
		switch (type){
		case TYPE_ACK:
			return new OnoMessageAck(data);
		case TYPE_RACE_RESULT:
			return new OnoMessageRaceResult(data);
		case TYPE_RACE:
		case TYPE_PRIMARY_PATH:
			return new OnoMessageData(data, type);		
		case TYPE_DEST_IP:
			return new OnoMessageDestIp(data);
		}
		
		return new OnoMessageInvalid(type);
	}
    
    public byte getType(){
    	return type;
    }
    
    public abstract NewTagByteBuffer toByteBuffer();

	@Override
	public String toString() {
		return getClass().getName();
	}
}
