/**
 * Ono Project
 *
 * File:         OnoMessageAck.java
 * RCS:          $Id: OnoMessageAck.java,v 1.3 2007/12/26 18:13:58 drc915 Exp $
 * Description:  OnoMessageAck class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 21, 2007 at 9:37:31 AM
 * Language:     Java
 * Package:      edu.northwestern.ono.net
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.net;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

import edu.northwestern.ono.connection.standard.StandardConnectionManager;
import edu.northwestern.ono.util.NewTagByteBuffer;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The OnoMessageAck class ...
 */
public class OnoMessageAck extends OnoMessage {
	
	byte hopCount=0;
	int ackNum;
	InetAddress ackPath;
	public final static int SIZE = 10;
	

	public OnoMessageAck(NewTagByteBuffer data) {
		tbb = data;
		bb = tbb.getBuffer();
		type = TYPE_ACK;
		bb.position(1);
		hopCount = bb.get();
		byte ackPathB[] = new byte[4];
		bb.get(ackPathB);
		try {
			ackPath = InetAddress.getByAddress(ackPathB);
		} catch (UnknownHostException e) {
			ackPath = null;
		}
		
		ackNum = bb.getInt();
//		System.out.println("Ack #"+ackNum+" for "+ackPath);                       
	}

	public OnoMessageAck() {
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see edu.northwestern.ono.net.OnoMessage#toByteBuffer()
	 */
	@Override
	public NewTagByteBuffer toByteBuffer() {
		
		bb.position(0);
		bb.put(TYPE_ACK);
		bb.put(hopCount);
		bb.put(ackPath.getAddress());
		bb.putInt(ackNum);
		bb.flip();
		return tbb;
	}
	
	public int getAckNum(){
		return ackNum;
	}
	
	public InetAddress getAckPath(){
		return ackPath;
	}
	
	public void setAckNum(int num){
		ackNum = num;
	}
	
	public void setAckPath(InetAddress path){
		ackPath = path;
	}

	public static OnoMessageAck create() {
		OnoMessageAck oma = new OnoMessageAck();
		oma.type = TYPE_ACK;
		oma.tbb = StandardConnectionManager.getInstance().getByteBuffer(SIZE);
		oma.bb = oma.tbb.getBuffer();
		return oma;
	}

}
