/**
 * Ono Project
 *
 * File:         OnoDataMessage.java
 * RCS:          $Id: OnoMessageData.java,v 1.5 2007/12/26 18:21:57 drc915 Exp $
 * Description:  OnoDataMessage class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 23, 2006 at 1:09:31 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.net;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

import edu.northwestern.ono.MainGeneric;
import edu.northwestern.ono.connection.IOnoConnection;
import edu.northwestern.ono.connection.standard.StandardConnectionManager;
import edu.northwestern.ono.util.NewTagByteBuffer;


/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The OnoDataMessage class represents a data message for data transfer.
 */
public class OnoMessageData extends OnoMessage {
    public static final int OVERHEAD = 1 + 1 + 4 + 4+2;
    byte hopCount=0;
    byte[] dest = null;
    int port;
    short seqNum;
    byte[] payload = null;
//    ByteBuffer bb;
	private boolean done;
    
	public OnoMessageData(){
		
	}

    public OnoMessageData(byte type, byte[] dest, int port) {
        this.type = type;

        if (dest != null) {
            this.dest = new byte[dest.length];
            System.arraycopy(dest, 0, this.dest, 0, dest.length);
        }

        this.port = port;
    }

    public OnoMessageData(NewTagByteBuffer data, byte type) {
    	this.type = type;
    	tbb = data;
    	bb = tbb.getBuffer();
    	bb.position(1);
    	hopCount = bb.get();
    	dest = new byte[4];
    	bb.get(dest);
    	port = bb.getInt();
    	seqNum = bb.getShort();
    	done = isDoneWithMiddle();

    	// leave payload until asked for
    	
	}
    
    public byte[] getPayload(){
    	if (payload==null){
	    	payload= new byte[bb.limit()-OVERHEAD];
	    	bb.position(OVERHEAD);
	    	bb.get(payload);
    	}
    	return payload;
    }
    
    private boolean isDoneWithMiddle() {
        if (bb.limit() == 18) { // done message is sent

            String s = "";
            bb.position(10);

            for (int i = 0; i < 4; i++)
                s += bb.getChar();

            if (s.equals("done")) {
                return true;
            }
        } else if (bb.limit() == 10) { // done message is sent

            String s = "";
            bb.position(2);

            for (int i = 0; i < 4; i++)
                s += bb.getChar();

            if (s.equals("done")) {
                return true;
            }
        }

        return false;
    }
    
    public boolean isDone(){
    	return done;
    }

	public static int payloadMaxSize(IOnoConnection connection) {
        // TODO Auto-generated method stub
    	if (connection==null) return StandardConnectionManager.getDefaultMessageSize()-OVERHEAD;
        return connection.getMaximumMessageSize() - OVERHEAD;
    }

    public void setData(byte[] bytesToSend) {
        
        tbb = MainGeneric.getConnectionManager().getByteBuffer(OVERHEAD + bytesToSend.length);
        bb = tbb.getBuffer();
        payload = bytesToSend;
    }

    public NewTagByteBuffer toByteBuffer() {
    	bb.position(0);
        bb.put(type);
        bb.put(hopCount);
        bb.put(dest);
        bb.putInt(port);
        bb.putShort(seqNum);
        bb.put(payload);
        bb.flip();

        return tbb;
    }    
    
    public byte getPacketType(){
    	return type;
    }

    public void setDest(InetAddress destIp){
    	dest = destIp.getAddress();
    }
    
    public InetAddress getDest(){
    	try {
			InetAddress destIp = InetAddress.getByAddress(dest);
			return destIp;
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	return null;
    }
    
    public void setPort(int port){
    	this.port = port;
    }
    
    public void setHopCount(int c){
    	hopCount = (byte)c;    	
    }

	public int getHopCount() {
		return hopCount;
	}

	public int getPort() {
		// TODO Auto-generated method stub
		return port;
	}

	public void setByteBuffer(ByteBuffer byteBuffer) {
		bb = byteBuffer;
		
	}

	public void setSeqNum(short seqNum) {
		this.seqNum = seqNum;
		
	}

	public int getSeqNum() {
		return seqNum;
	}

    

//	protected void finalize() throws Throwable {
//		MainGeneric.getConnectionManager().returnToPool(bb);
//	}
    
    
}
