/**
 * Ono Project
 *
 * File:         OnoMessageDestIp.java
 * RCS:          $Id: OnoMessageDestIp.java,v 1.2 2007/12/26 18:13:58 drc915 Exp $
 * Description:  OnoMessageDestIp class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 21, 2007 at 11:05:28 AM
 * Language:     Java
 * Package:      edu.northwestern.ono.net
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.net;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

import edu.northwestern.ono.connection.standard.StandardConnectionManager;
import edu.northwestern.ono.util.NewTagByteBuffer;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The OnoMessageDestIp class ...
 */
public class OnoMessageDestIp extends OnoMessage {

	InetAddress ip;
	short port;
	
	public OnoMessageDestIp(NewTagByteBuffer data) {
		this.tbb = data;
		bb = data.getBuffer();
		bb.position(1);
		byte ipBytes[] = new byte[4];
		bb.get(ipBytes);
		try {
			ip = InetAddress.getByAddress(ipBytes);
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			ip = null;
		}
		port = bb.getShort();
	}

	public OnoMessageDestIp() {
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see edu.northwestern.ono.net.OnoMessage#toByteBuffer()
	 */
	@Override
	public NewTagByteBuffer toByteBuffer() {
		if (bb==null){
			tbb = StandardConnectionManager.getInstance().getByteBuffer(7);
			bb = tbb.getBuffer();
		}
		bb.position(0);
		bb.put(TYPE_DEST_IP);
		bb.put(ip.getAddress());
		bb.putShort(port);
		bb.flip();
		return tbb;
	}

	public InetAddress getIp() {
		return ip;
	}

	public void setIp(InetAddress ip) {
		this.ip = ip;
	}

	public short getPort() {
		return port;
	}

	public void setPort(short port) {
		this.port = port;
	}

	public static OnoMessageDestIp create() {
		OnoMessageDestIp omdi = new OnoMessageDestIp();
		omdi.bb = null;		
		omdi.type = TYPE_DEST_IP;
		return omdi;
	}

}
