/**
 * Ono Project
 *
 * File:         OnoMessageRaceResult.java
 * RCS:          $Id: OnoMessageRaceResult.java,v 1.2 2007/12/26 18:13:58 drc915 Exp $
 * Description:  OnoMessageRaceResult class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 21, 2007 at 10:27:51 AM
 * Language:     Java
 * Package:      edu.northwestern.ono.net
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.net;

import java.awt.List;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;

import org.xbill.DNS.Address;

import edu.northwestern.ono.connection.standard.StandardConnectionManager;
import edu.northwestern.ono.experiment.OnoExperiment;
import edu.northwestern.ono.util.NewTagByteBuffer;
import edu.northwestern.ono.util.Pair;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The OnoMessageRaceResult class ...
 */
public class OnoMessageRaceResult extends OnoMessage {

	private int numEntries;
	LinkedList<Pair<Double, InetAddress>> raceResult;

	public OnoMessageRaceResult(NewTagByteBuffer data) {		
		tbb= data;
		bb = tbb.getBuffer();
		bb.position(0);
		type = bb.get();
		numEntries = bb.getInt();         
	}

	public OnoMessageRaceResult() {
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see edu.northwestern.ono.net.OnoMessage#toByteBuffer()
	 */
	@Override
	public NewTagByteBuffer toByteBuffer() {
		if (bb==null){
			tbb = StandardConnectionManager.getInstance().getByteBuffer(
					1+numEntries*(Double.SIZE/8+4));
			bb = tbb.getBuffer();
		}
		bb.position(0);
		bb.put(TYPE_RACE_RESULT);
		for (Pair<Double, InetAddress> p : raceResult){
			bb.put(p.getValue().getAddress());
			bb.putDouble(p.getKey());
		}
		bb.flip();
		return tbb;
	}
	
	public void parseRaceResults(OnoExperiment thisExp, String myIp){
		byte ipBytes[] = new byte[4];
		double[] races = new double[numEntries];
		bb.position(5);

        raceResult = new LinkedList<Pair<Double, InetAddress>>();


        System.out.println("Race Results: ");
        for (int i = 0; i < numEntries; i++) {
            // get ip address
            bb.get(ipBytes);

            try {
                InetAddress curAddr = InetAddress.getByAddress(ipBytes);

                if (curAddr.toString().substring(1).equals(myIp)) {
                    curAddr = Address.getByAddress(thisExp.dest);
                }
                races[i] = bb.getDouble();
                raceResult.add(new Pair<Double, InetAddress>(races[i], curAddr));
                System.out.println("\t"+curAddr+"\t"+races[i]);
            } catch (UnknownHostException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
		Collections.sort(raceResult, new Comparator<Pair<Double, InetAddress>>(){

			public int compare(Pair<Double, InetAddress> o1,
					Pair<Double, InetAddress> o2) {				
				return o1.getKey().compareTo(o2.getKey());
			}});
	}
	
	public LinkedList<Pair<Double, InetAddress>> getRaceResults(){

		return raceResult;
	}
	
	public void addRaceResult(InetAddress ip, double rate){
		if (raceResult == null){
			raceResult = new LinkedList<Pair<Double, InetAddress>>();
		}
		
		raceResult.add(new Pair<Double, InetAddress>(rate, ip));
		numEntries++;
		
	}

	public static OnoMessageRaceResult create() {
		OnoMessageRaceResult omrr = new OnoMessageRaceResult();
		omrr.bb = null; 
		omrr.type = TYPE_RACE_RESULT;
		omrr.numEntries = 0;
		return omrr;
	}

}
